##
## library.py
## Login : <glibersat@localhost.localdomain>
## Started on  Fri Jul 22 17:42:41 2005 Guillaume Libersat
## $Id$
## 
## Copyright (C) 2005 Guillaume Libersat
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

from preferences import *

from Crypto.Hash import MD5

import gtk, gobject, dircache, gzip, os

class LibraryManager:
    def load(self, path):

        # Once again, try to open as gziped file, then regular
        fd = gzip.GzipFile(path, "r", compresslevel=9)
        try:
            fd.readline()
            fd.rewind()
        except:
            fd = open(path, "r")
        
        tab = ""
        for line in fd.readlines():
            tab += line

        off = tab.find("%%>\n") + 4

        tab = tab[off:]
        
        fd.close()
        
        return tab

    def save(self, tabinfo, tab):
        pref = Preferences()
        library_path = pref.get_library_path()

        md5sum = MD5.new()
        for el in tabinfo:
            if el != None:
                md5sum.update(el)

        path = os.path.join(library_path, md5sum.hexdigest())
        # fd = gzip.GzipFile(path, "w", compresslevel=9)
        fd = open(path, "w")

        fd.write("<%%\n")
        fd.write("Artist: %s\n" % tabinfo[0])
        fd.write("Title: %s\n" % tabinfo[1])
        fd.write("Version: %s\n" % tabinfo[2])
        fd.write("Type: %s\n" % tabinfo[3])
        fd.write("Rate: %s\n" % tabinfo[4])
        fd.write("Source: %s\n" % tabinfo[5])
        fd.write("%%>\n")
        fd.write(tab)

        fd.close()

    def delete(self,tabinfo):
        path = tabinfo[7]
        os.unlink(path)

class Library:
    def __init__(self):
        pref = Preferences()
        self.library_path = pref.get_library_path()
        
        self.data = list()


    def refresh(self):
        dir = dircache.opendir(self.library_path)
        
        self.data = list()
        
        for file in dir:
            # Reset fields
            artist = None
            title = None
            version = None
            type = None
            rate = None
            source = None

            # Build file path
            path = os.path.join(self.library_path, file)

            # Open it as gzip file, and if fails, open as regular file
            # It sucks a bit because exception can be caught on readline only
            fd = gzip.GzipFile(path, "r", compresslevel=9)
            try:
                fd.readline()
                fd.rewind()
            except:
                fd = open(path, "r")
            
            while 1:
                line = fd.readline()
                if line == "%%>\n" or line == "":
                    break
                else:
                    if line.startswith("Artist"):
                        artist = line.split(" ", 1)[1]
                        artist = artist.rstrip("\n")
                    elif line.startswith("Title"):
                        title = line.split(" ", 1)[1]
                        title = title.rstrip("\n")
                    elif line.startswith("Version"):
                        version = line.split(" ", 1)[1]
                        version = version.rstrip("\n")
                    elif line.startswith("Type"):
                        type = line.split(" ", 1)[1]
                        type = type.rstrip("\n")
                    elif line.startswith("Rate"):
                        rate = line.split(" ", 1)[1]
                        rate = rate.rstrip("\n")
                    elif line.startswith("Source"):
                        source = line.split(" ", 1)[1]
                        source = source.rstrip("\n")

            if artist != None and (title != None or version != None or type != None or rate != None or source != None):

                where = None
                for el in self.data:
                    if el[0] == artist:
                        where = el

                # We don't have this artist yet, create it
                if where == None:
                    l = list()
                    self.data.append((artist, l))
                    
                # Just append title to artist's song list
                else:
                    l = where[1]

                # Append title
                l.append((title, version, type, rate, source, path))

            # Sort all data (artists, then by title, version, ...)
            self.data.sort()
            
            for art in self.data:
                art[1].sort()

            fd.close()

    def get_songs_from_artist(self, artist, filter = None):
        good = None
        
        for art in self.data:
            if art[0] == artist:
                good = art
                break

        if good == None:
            return None

        # Build song list
        songs_list = list()
        for el in good[1]:
            title, version, type, rate, source, path = el
            
            if filter != None:
                if title.lower().find(filter.lower()) != -1:
                    entry = (good[0], title, version, type, rate, source, "library", path)
                    songs_list.append(entry)
            else:
                entry = (good[0], title, version, type, rate, source, "library", path)
                songs_list.append(entry)

        return songs_list

    def get_all_songs(self, songfilter = None, artistfilter = None):
        songs_list = list()

        for art in self.data:
            res = self.get_songs_from_artist(art[0], songfilter)
            if res != None:
                songs_list.extend(res)
        
        return songs_list
    
    def add(self, tabinfo, tab):
        lm = LibraryManager()
        lm.save(tabinfo, tab)

    def get_list(self):
        return self.data

    
class LibrarySelector(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self, 2)
        self.set_homogeneous(False)
        
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_ETCHED_IN)


        self.tree = gtk.TreeView(self.__create_model())
        sel = self.tree.get_selection()
        sel.set_mode(gtk.SELECTION_SINGLE)
        
        self.__add_columns()

        ref_button = gtk.Button("Refresh", gtk.STOCK_REFRESH)
        ref_button.connect("clicked", self.__refresh_callback)

        scroll.add(self.tree)
        self.pack_start(scroll, True, True, 0)
        self.pack_start(ref_button, False, True, 0)
        
        self.tree.connect("cursor-changed", self.artist_changed)
        
        self.library = Library()

        self.refresh()


    def __refresh_callback(self, data):
        self.refresh()

        self.get_toplevel().statusbar_push("Library refreshed")
        

    def refresh(self):

        # remember what was selected
        selection = self.tree.get_selection()
        init_model, init_path = selection.get_selected_rows()
        
        if init_path:
            tr = gtk.TreeRowReference(init_model,init_path[0])
        else:
            tr = None

        self.library.refresh()

        data = list()
        data.append(("All", None))
        data.extend(self.library.get_list())

        model = self.tree.get_model()
        model.clear()

        if data != None:
            for item in data:
                iter = model.append()
                model.set(iter,
                          0, item[0],
                          1, item[1]
                          )

        if not tr:
            selection.select_iter(model.get_iter_root())


    def __add_columns(self):
        model = self.tree.get_model()

        
        column = gtk.TreeViewColumn("Artist", gtk.CellRendererText(), text=0)
        self.tree.append_column(column)

    def __create_model(self):
        lstore = gtk.ListStore(
            gobject.TYPE_STRING,
            gobject.TYPE_STRING
            )

        return lstore


    def artist_changed(self, action):
        ''' When we select an artist from the left list '''
        songs = None
        artist = None
        songinfo = None
        
        if self.get() != None:
            artist, songinfo = self.get()

        if artist != None:
            self.get_toplevel().statusbar_push("Artist \"%s\" selected" % artist)

        # If songinfo is None, this means we want all songs
        if songinfo != None:
            songs = self.library.get_songs_from_artist(artist)
        else:
            songs = self.library.get_all_songs()
            
        self.get_toplevel().main_table.resultview.update(songs)


    def get(self):
        ''' Return which artist is selected '''
        
        selection = self.tree.get_selection()
        
        model, iter = selection.get_selected()
        
        if iter:
            artist = model.get_value(iter, 0)
            song = model.get_value(iter, 1)
            return (artist, song)
        else:
            return None

    def query(self, query):
        artist, songs = self.get()

        if artist == "All" and songs == None:
            song_list = self.library.get_all_songs(query)
        else:
            song_list = self.library.get_songs_from_artist(artist, query)

        return song_list




        
