##
## search.py
## Login : <glibersat@localhost.localdomain>
## Started on  Tue Jul 19 15:37:58 2005 Guillaume Libersat
## $Id$
## 
## Copyright (C) 2005 Guillaume Libersat
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import gtk
from plugins import *

( LOCAL_MODE,
  NETWORK_MODE ) = range(2)

class SearchBox(gtk.HBox):
    def __init__(self, update_method):
        gtk.HBox.__init__(self, False, 2)

        self.field = gtk.Entry()
        self.field.connect("activate", self.__search, update_method)
        self.field.grab_focus()
        self.pack_start(self.field, True, True, 0)
        
        self.search_button = gtk.Button(stock="gtk-find")
        self.search_button.connect("clicked", self.__search, update_method)
        self.pack_start(self.search_button, False, False, 1)

        self.clear_button = gtk.Button(stock="gtk-clear")
        self.clear_button.connect("clicked", self.__clear, update_method)
        self.pack_start(self.clear_button, False, False, 1)
        
        self.mode = LOCAL_MODE

    def is_in_local_mode(self):
        return self.mode == LOCAL_MODE

    def set_local_mode(self):
        self.mode = LOCAL_MODE

    def set_network_mode(self):
        self.mode = NETWORK_MODE

    def __clear(self, button, update_method):
        self.get_toplevel().statusbar_push("List cleared")
        update_method(None)
        self.field.grab_focus()
        

    def __search(self, button, update_method):
        # This is what the user entered in the search field
        query = self.field.get_text()

        
        # Get which source/artist is selected depending on mode
        if self.mode == LOCAL_MODE:
            artist = self.get_toplevel().main_table.librarysel.get()
            res = self.__search_library(query, artist, update_method)
        elif self.mode == NETWORK_MODE:
            source = self.get_toplevel().main_table.sourcesel.get()
            res = self.__search_network(query, source, update_method)

        if res > 0:
            self.get_toplevel().statusbar_push("Found %d result(s)" % res)
        else:
            self.get_toplevel().statusbar_push("No result found")

        # Select whole text, and give focus to query entry
        self.field.select_region(0, len(self.field.get_text()))
        self.field.grab_focus()


    def __search_library(self, query, artist, update_method):
        # No artist is selected
        if artist == None:
            self.get_toplevel().statusbar_push("Please select an artist !")
        else:
            # Getting results from library
            if query == "":
                self.get_toplevel().statusbar_push("Listing songs from artist %s..." % artist[0])
            else:
                self.get_toplevel().statusbar_push("Searching %s..." % query)
            
        # Empty query
        if query == "":
            query = None
            
        data = self.get_toplevel().main_table.librarysel.query(query)
        update_method(data)

        return len(data)


    def __search_network(self, query, source, update_method):
        # Empty query
        if query == "":
            self.get_toplevel().statusbar_push("Please enter a band or title name")

        # No source is selected
        elif source == None:
            self.get_toplevel().statusbar_push("Please select a source !")

        else:
            # Getting results from website
            self.get_toplevel().statusbar_push("Searching %s..." % query)

            # "All networks" was selected
            if source[1] == "__all__":
                data = list()
                all_sources = self.get_toplevel().main_table.sourcesel.get_all()
                for source in all_sources:
                    plug = PluginLoader(source)
                    data.extend(plug.query(query))
                    

            # One network was selected
            else:
                plug = PluginLoader(source[1])
                data = plug.query(query)

            # Update results list
            update_method(data)

            return len(data)



        
