##
## source.py
## Login : <glibersat@localhost.localdomain>
## Started on  Thu Jul 21 16:22:46 2005 Guillaume Libersat
## $Id$
## 
## Copyright (C) 2005 Guillaume Libersat
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import gtk, gobject
from plugins import PluginLister, PluginLoader


class SourceSelector(gtk.ScrolledWindow):
    def __init__(self):
        gtk.ScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        self.set_size_request(150, -1)

        self.tree = gtk.TreeView(self.__create_model())
        sel = self.tree.get_selection()
        sel.set_mode(gtk.SELECTION_SINGLE)
        
        self.__add_columns()
        self.refresh()
        
        self.add(self.tree)

        self.tree.connect("cursor-changed", self.__source_changed)


    def __source_changed(self, action):
        src = self.get()

        if src != None:
            self.get_toplevel().statusbar_push("Source \"%s\" selected" % src[0])

        
    def get(self):
        ''' Return which source is selected '''
        
        selection = self.tree.get_selection()
        
        model, iter = selection.get_selected()
        
        if iter:
            name = model.get_value(iter, 0)
            code = model.get_value(iter, 1)
            return (name, code)
        else:
            return None

    def get_all(self):
        ''' Return all sources except "All Networks" '''
        
        model = self.tree.get_model()
        iter = model.get_iter_first()

        source = list()

        while model.iter_next(iter) != None:
            iter = model.iter_next(iter)
            source.append(model.get_value(iter, 1))

        return source

    def refresh(self):

        data = list()

        data.append(("All Networks", "__all__"))

        # Append plugin names to list
        pl = PluginLister()
        data.extend(pl.get_list())
        
        model = self.tree.get_model()
        model.clear()

        if data != None:
            for item in data:
                iter = model.append()
                model.set(iter,
                          0, item[0],
                          1, item[1]
                          )
                
    def __add_columns(self):
        model = self.tree.get_model()

        
        column = gtk.TreeViewColumn("Source", gtk.CellRendererText(), text=0)
        self.tree.append_column(column)

    def __create_model(self):
        lstore = gtk.ListStore(
            gobject.TYPE_STRING,
            gobject.TYPE_STRING
            )

        return lstore

